/**********************************************************************
Copyright (c) 2009 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
   ...
**********************************************************************/
package org.datanucleus.jpa.criteria;

import java.util.HashSet;
import java.util.Set;

import javax.persistence.criteria.CollectionJoin;
import javax.persistence.criteria.Fetch;
import javax.persistence.criteria.From;
import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.ListJoin;
import javax.persistence.criteria.MapJoin;
import javax.persistence.criteria.SetJoin;
import javax.persistence.metamodel.CollectionAttribute;
import javax.persistence.metamodel.ListAttribute;
import javax.persistence.metamodel.ManagedType;
import javax.persistence.metamodel.MapAttribute;
import javax.persistence.metamodel.PluralAttribute;
import javax.persistence.metamodel.SetAttribute;
import javax.persistence.metamodel.SingularAttribute;

import org.datanucleus.jpa.metamodel.AttributeImpl;
import org.datanucleus.jpa.metamodel.CollectionAttributeImpl;
import org.datanucleus.jpa.metamodel.ListAttributeImpl;
import org.datanucleus.jpa.metamodel.MapAttributeImpl;
import org.datanucleus.jpa.metamodel.SetAttributeImpl;
import org.datanucleus.jpa.metamodel.SingularAttributeImpl;

/**
 * Implementation of JPA2 Criteria "From".
 * 
 * @param <Z>
 * @param <X>
 */
public class FromImpl<Z,X> extends PathImpl<Z,X> implements From<Z,X>
{
    protected java.util.Set<Join<X, ?>> joins;
    protected java.util.Set<Fetch<X, ?>> fetchJoins; 

    public FromImpl(ManagedType<X> type)
    {
        super(type.getJavaType());
    }

    public FromImpl(PathImpl<?, Z> parent, AttributeImpl<? super Z, ?> type)
    {
        super(parent, type, (Class<X>) type.getJavaType());
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#getJoins()
     */
    public Set<Join<X, ?>> getJoins()
    {
        return joins;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.CollectionAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> CollectionJoin<X, Y> join(CollectionAttribute<? super X, Y> attr, JoinType joinType)
    {
        CollectionJoin<X, Y> join = new CollectionJoinImpl<X, Y>((FromImpl)parent, (CollectionAttributeImpl) attr, joinType);
        if (joins == null)
        {
            joins = new HashSet<Join<X,?>>();
        }
        joins.add(join);
        return join;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.CollectionAttribute)
     */
    public <Y> CollectionJoin<X, Y> join(CollectionAttribute<? super X, Y> collection)
    {
        return join(collection, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.ListAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> ListJoin<X, Y> join(ListAttribute<? super X, Y> attr, JoinType joinType)
    {
        ListJoin<X, Y> join = new ListJoinImpl<X, Y>((FromImpl)parent, (ListAttributeImpl) attr, joinType);
        if (joins == null)
        {
            joins = new HashSet<Join<X,?>>();
        }
        joins.add(join);
        return join;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.ListAttribute)
     */
    public <Y> ListJoin<X, Y> join(ListAttribute<? super X, Y> list)
    {
        return join(list, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.MapAttribute, javax.persistence.criteria.JoinType)
     */
    public <K, V> MapJoin<X, K, V> join(MapAttribute<? super X, K, V> attr, JoinType joinType)
    {
        MapJoin<X, K, V> join = new MapJoinImpl<X, K, V>((FromImpl)parent, (MapAttributeImpl) attr, joinType);
        if (joins == null)
        {
            joins = new HashSet<Join<X,?>>();
        }
        joins.add(join);
        return join;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.MapAttribute)
     */
    public <K, V> MapJoin<X, K, V> join(MapAttribute<? super X, K, V> map)
    {
        return join(map, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.SetAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> SetJoin<X, Y> join(SetAttribute<? super X, Y> attr, JoinType joinType)
    {
        SetJoin<X, Y> join = new SetJoinImpl<X, Y>((FromImpl)parent, (SetAttributeImpl) attr, joinType);
        if (joins == null)
        {
            joins = new HashSet<Join<X,?>>();
        }
        joins.add(join);
        return join;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.SetAttribute)
     */
    public <Y> SetJoin<X, Y> join(SetAttribute<? super X, Y> set)
    {
        return join(set, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.SingularAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> Join<X, Y> join(SingularAttribute<? super X, Y> attr, JoinType joinType)
    {
        Join<X, Y> join = new JoinImpl<X, Y>(this, (SingularAttributeImpl<X, Y>) attr, joinType);
        if (joins == null)
        {
            joins = new HashSet<Join<X, ?>>();
        }
        joins.add(join);
        return join;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(javax.persistence.metamodel.SingularAttribute)
     */
    public <Y> Join<X, Y> join(SingularAttribute<? super X, Y> singular)
    {
        return join(singular, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, Y> Join<X, Y> join(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#join(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, Y> Join<X, Y> join(String attr)
    {
        return join(attr, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinCollection(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, Y> CollectionJoin<X, Y> joinCollection(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinCollection(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, Y> CollectionJoin<X, Y> joinCollection(String attr)
    {
        return joinCollection(attr, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinList(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, Y> ListJoin<X, Y> joinList(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinList(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, Y> ListJoin<X, Y> joinList(String attr)
    {
        return joinList(attr, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinMap(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, K, V> MapJoin<X, K, V> joinMap(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinMap(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, K, V> MapJoin<X, K, V> joinMap(String attr)
    {
        return joinMap(attr, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinSet(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, Y> SetJoin<X, Y> joinSet(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.From#joinSet(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, Y> SetJoin<X, Y> joinSet(String attr)
    {
        return joinSet(attr, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#getFetches()
     */
    public Set<Fetch<X, ?>> getFetches()
    {
        return fetchJoins;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(javax.persistence.metamodel.PluralAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> Fetch<X, Y> fetch(PluralAttribute<? super X, ?, Y> plural, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(javax.persistence.metamodel.PluralAttribute)
     */
    public <Y> Fetch<X, Y> fetch(PluralAttribute<? super X, ?, Y> plural)
    {
        return fetch(plural, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(javax.persistence.metamodel.SingularAttribute, javax.persistence.criteria.JoinType)
     */
    public <Y> Fetch<X, Y> fetch(SingularAttribute<? super X, Y> singular, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(javax.persistence.metamodel.SingularAttribute)
     */
    public <Y> Fetch<X, Y> fetch(SingularAttribute<? super X, Y> singular)
    {
        return fetch(singular, JoinType.INNER);
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(java.lang.String, javax.persistence.criteria.JoinType)
     */
    @SuppressWarnings("hiding")
    public <X, Y> Fetch<X, Y> fetch(String attr, JoinType joinType)
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.FetchParent#fetch(java.lang.String)
     */
    @SuppressWarnings("hiding")
    public <X, Y> Fetch<X, Y> fetch(String attr)
    {
        return fetch(attr, JoinType.INNER);
    }
}