/**********************************************************************
Copyright (c) 2009 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
   ...
**********************************************************************/
package org.datanucleus.jpa.criteria;

import java.util.Iterator;

import javax.persistence.criteria.Join;
import javax.persistence.criteria.JoinType;
import javax.persistence.criteria.Root;
import javax.persistence.metamodel.Attribute;
import javax.persistence.metamodel.EntityType;

/**
 * Implementation of JPA2 Criteria "Root".
 */
public class RootImpl<X> extends FromImpl<X, X> implements Root<X>
{
    private final EntityType<X> entity;

    public RootImpl(EntityType<X> type)
    {
        super(type);
        this.entity = type;
    }

    public EntityType<X> getModel()
    {
        return entity;
    }

    /**
     * Method to return a string form of the root expression.
     * @return The string form
     */
    public String toString()
    {
        return toString(false);
    }

    /**
     * Method to return a string form of the root expression.
     * @param from Whether this is for the FROM clause
     * @return The string form
     */
    public String toString(boolean from)
    {
        if (from)
        {
            // "mydomain.MyClass {alias} JOIN ..."
            StringBuffer str = new StringBuffer();
            str.append(getJavaType().getName());
            if (getAlias() != null)
            {
                str.append(" ").append(getAlias());
            }

            if (joins != null)
            {
                Iterator<Join<X, ?>> iter = joins.iterator();
                while (iter.hasNext())
                {
                    Join<X, ?> join = iter.next();
                    JoinType type = join.getJoinType();
                    if (type == JoinType.INNER)
                    {
                        str.append("JOIN ");
                    }
                    else if (type == JoinType.LEFT)
                    {
                        str.append("LEFT JOIN ");
                    }
                    else if (type == JoinType.RIGHT)
                    {
                        str.append("RIGHT JOIN ");
                    }

                    Attribute<? super X, ?> attr = join.getAttribute();
                    str.append(attr);

                    if (join.getAlias() != null)
                    {
                        str.append(join.getAlias());
                    }
                    if (iter.hasNext())
                    {
                        str.append(" ");
                    }
                }
            }

            if (fetchJoins != null)
            {
                // TODO Process fetch joins
            }

            return str.toString();
        }
        else
        {
            // "{alias}"
            return getAlias();
        }
    }
}