/**********************************************************************
Copyright (c) 2009 Andy Jefferson and others. All rights reserved.
Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

Contributors:
   ...
**********************************************************************/
package org.datanucleus.jpa.criteria;

import java.util.List;

import javax.persistence.criteria.Selection;

/**
 * Implementation of JPA2 Criteria "Selection".
 * An item selected in the projection clause of Criteria query.
 * Base implementation for all concrete expressions.
 * @param <X> the type this term evaluates to 
 */
public class SelectionImpl<X> implements Selection<X>
{
    private final Class<X> cls;
    private String alias;

    /**
     * Constructor, setting the immutable type that the select is for.
     * @param cls The type
     */
    public SelectionImpl(Class<X> cls)
    {
        this.cls = cls;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.Selection#alias(java.lang.String)
     */
    public Selection<X> alias(String alias)
    {
        this.alias = alias;
        return this;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.Selection#getCompoundSelectionItems()
     */
    public List<Selection<?>> getCompoundSelectionItems()
    {
        // TODO Auto-generated method stub
        return null;
    }

    /* (non-Javadoc)
     * @see javax.persistence.criteria.Selection#isCompoundSelection()
     */
    public boolean isCompoundSelection()
    {
        // TODO Auto-generated method stub
        return false;
    }

    /* (non-Javadoc)
     * @see javax.persistence.TupleElement#getAlias()
     */
    public String getAlias()
    {
        return alias;
    }

    /* (non-Javadoc)
     * @see javax.persistence.TupleElement#getJavaType()
     */
    public Class<? extends X> getJavaType()
    {
        return cls;
    }
}